<?php

namespace Fazed\NyaaRssParser\Models;

use ArrayAccess;
use Illuminate\Contracts\Support\Jsonable;
use Illuminate\Contracts\Support\Arrayable;
use Fazed\NyaaRssParser\Contracts\Models\EntryContract;

class Entry implements EntryContract, Arrayable, ArrayAccess, Jsonable
{
    /**
     * @var string
     */
    private $title;

    /**
     * @var string
     */
    private $description;

    /**
     * @var string
     */
    private $link;

    /**
     * @var string
     */
    private $guid;

    /**
     * @var string
     */
    private $pubDate;

    /**
     * @var int
     */
    private $seeders;

    /**
     * @var int
     */
    private $leechers;

    /**
     * @var int
     */
    private $downloads;

    /**
     * @var string
     */
    private $infoHash;

    /**
     * @var string
     */
    private $categoryId;

    /**
     * @var string
     */
    private $category;

    /**
     * @var string
     */
    private $size;

    /**
     * Entry constructor.
     *
     * @param array $properties
     */
    public function __construct(array $properties = [])
    {
        $this->fill($properties);
    }

    /**
     * {@inheritdoc}
     */
    public function fill(array $properties)
    {
        foreach ($properties as $property=>$value) {
            $this->offsetSet($property, $value);
        }

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * {@inheritdoc}
     */
    public function setTitle($title)
    {
        $this->title = $title;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * {@inheritdoc}
     */
    public function setDescription($description)
    {
        $this->description = $description;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getLink()
    {
        return $this->link;
    }

    /**
     * {@inheritdoc}
     */
    public function setLink($link)
    {
        $this->link = $link;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getGuid()
    {
        return $this->guid;
    }

    /**
     * {@inheritdoc}
     */
    public function setGuid($guid)
    {
        $this->guid = $guid;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getPubDate()
    {
        return $this->pubDate;
    }

    /**
     * {@inheritdoc}
     */
    public function setPubDate($pubDate)
    {
        $this->pubDate = $pubDate;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getSeeders()
    {
        return $this->seeders;
    }

    /**
     * {@inheritdoc}
     */
    public function setSeeders($seeders)
    {
        $this->seeders = $seeders;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getLeechers()
    {
        return $this->leechers;
    }

    /**
     * {@inheritdoc}
     */
    public function setLeechers($leechers)
    {
        $this->leechers = $leechers;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getDownloads()
    {
        return $this->downloads;
    }

    /**
     * {@inheritdoc}
     */
    public function setDownloads($downloads)
    {
        $this->downloads = $downloads;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getInfoHash()
    {
        return $this->infoHash;
    }

    /**
     * {@inheritdoc}
     */
    public function setInfoHash($infoHash)
    {
        $this->infoHash = $infoHash;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getCategoryId()
    {
        return $this->categoryId;
    }

    /**
     * {@inheritdoc}
     */
    public function setCategoryId($categoryId)
    {
        $this->categoryId = $categoryId;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getCategory()
    {
        return $this->category;
    }

    /**
     * {@inheritdoc}
     */
    public function setCategory($category)
    {
        $this->category = $category;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getSize()
    {
        return $this->size;
    }

    /**
     * {@inheritdoc}
     */
    public function setSize($size)
    {
        $this->size = $size;
        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function toArray()
    {
        return get_object_vars($this);
    }

    /**
     * {@inheritdoc}
     */
    public function offsetExists($offset)
    {
        return property_exists($this, $offset);
    }

    /**
     * {@inheritdoc}
     */
    public function offsetGet($offset)
    {
        if ($this->offsetExists($offset)) {
            return $this->$offset;
        }

        return null;
    }

    /**
     * {@inheritdoc}
     */
    public function offsetSet($offset, $value)
    {
        if ($this->offsetExists($offset)) {
            $this->$offset = $value;
        }
    }

    /**
     * {@inheritdoc}
     */
    public function offsetUnset($offset)
    {
        if ($this->offsetExists($offset)) {
            unset($this->$offset);
        }
    }

    /**
     * {@inheritdoc}
     */
    public function toJson($options = 0)
    {
        return json_encode($this->toArray(), $options);
    }
}
